<?php

namespace EasyContent\WordPress\Implementation\Db;

use EasyContent\WordPress\Implementation\Db\Migrations\ArticleCacheTableUpdateMigration;
use EasyContent\WordPress\Implementation\Db\Migrations\ArticleTitlesTypeFixMigration;
use EasyContent\WordPress\Implementation\Db\Migrations\InitialMigration;
use EasyContent\WordPress\Implementation\Entity\Db\Version;

final class Migrator
{
    const CONFIG = [
        '1.0.0' => InitialMigration::class,
        '1.0.1' => ArticleTitlesTypeFixMigration::class,
        '1.0.2' => ArticleCacheTableUpdateMigration::class,
    ];

    /** @var Version $dbVersion */
    private $dbVersion;
    /** @var Version $pluginVersion */
    private $pluginVersion;

    /**
     * @param Version $dbVersion
     * @param Version $pluginVersion
     */
    public function __construct(Version $dbVersion, Version $pluginVersion)
    {
        $this->dbVersion = $dbVersion;
        $this->pluginVersion = $pluginVersion;
    }

    public function migrate()
    {
        $this->ensureMigrationNeeded();

        $versions = array_keys(self::CONFIG);
        $migrations = array_values(self::CONFIG);

        $cvi = $this->dbVersion->isDefault() // Current version index
            ? -1
            : array_search($this->dbVersion->asString(), $versions, true);

        if (false === $cvi) {
            throw new \Exception("Unable to find migration class for {$this->dbVersion->asString()} version");
        }

        $fvi = array_search($this->pluginVersion->asString(), $versions, true); // Fresh version index

        if (false === $fvi) {
            throw new \Exception("Unable to find migration class for {$this->pluginVersion->asString()} version");
        }

        $offset = $cvi + 1;
        $length = $fvi - $cvi;

        if (!$toRun = array_slice($migrations, $offset, $length)) {
            return;
        }

        foreach ($toRun as $migrationClass) {
            (new $migrationClass)->run();
        }
    }

    private function ensureMigrationNeeded()
    {
        if ($this->pluginVersion->greaterThan($this->dbVersion)) { // Need migration
            return;
        }

        $message = sprintf(
            "Couldn't migrate from '%s' to '%s'",
            $this->dbVersion->asString(),
            $this->pluginVersion->asString()
        );
        throw new \DomainException($message);
    }
}
