<?php

namespace EasyContent\WordPress\Implementation\Entity;

use EasyContent\WordPress\Source\Entity\CachedArticle;
use EasyContent\WordPress\Source\Entity\CachedArticlesStorage;
use EasyContent\WordPress\Source\Exception\CachedArticleNotFoundException;

final class DbCachedArticlesStorage implements CachedArticlesStorage
{
    private $tableName;
    private $junctionTableName;

    public function __construct()
    {
        global $wpdb;
        $this->tableName = "{$wpdb->prefix}easycontent_article";
        $this->junctionTableName = "{$wpdb->prefix}easycontent_article_post";
    }

    public function removeByIds(array $ids)
    {
        if (!$ids) {
            return;
        }

        global $wpdb;
        $chunks = array_chunk($ids, 500);

        foreach ($chunks as $chunk) {
            $idsStr = implode(', ', $chunk);

            $query = "DELETE FROM `{$this->tableName}` WHERE `id` IN ({$idsStr});";
            $wpdb->query($query);

            $query = "DELETE FROM `{$this->junctionTableName}` WHERE `article_id` IN ({$idsStr});";
            $wpdb->query($query);
        }
    }

    /**
     * @return array|CachedArticle[]
     */
    public function all()
    {
        global $wpdb;
        $query = "SELECT * FROM `{$this->tableName}`;";
        $rows = $wpdb->get_results($query, ARRAY_A);

        if (!is_array($rows)) {
            return [];
        }

        return array_map([$this, 'fromRow'], $rows);
    }

    /**
     * @param array|CachedArticle[] $items
     * @return void
     */
    public function batchDelete(array $items)
    {
        $ids = array_map(function(CachedArticle $c) { return $c->getId(); }, $items);
        $this->removeByIds($ids);
    }

    /**
     * @param CachedArticle $item
     * @return void
     */
    public function insert($item)
    {
        global $wpdb;
        $wpdb->insert(
            $this->tableName,
            $this->toArray($item)
        );
    }

    /**
     * @param CachedArticle $item
     * @return void
     */
    public function update($item)
    {
        global $wpdb;
        $wpdb->update(
            $this->tableName,
            $this->toArray($item),
            ['id' => $item->getId()]
        );
    }

    /**
     * Maps from DB value to PHP value
     *
     * @param array $row
     * @return CachedArticle
     */
    private function fromRow(array $row)
    {
        return CachedArticle::fromState([
            'id' => (int)$row['id'],
            'current_workflow_stage_id' => (int)$row['current_workflow_stage_id'],
            'topic_name' => $row['topic_name'],
            'title' => $row['title'],
            'updated' => $row['updated'] === null ? null : (int)$row['updated'],
        ]);
    }

    /**
     * Maps from PHP value to DB value
     *
     * @param CachedArticle $article
     * @return array
     */
    private function toArray(CachedArticle $article)
    {
        return [
            'id' => $article->getId(),
            'current_workflow_stage_id' => $article->getCurrentWorkflowStatusId(),
            'topic_name' => $article->getTopicName(),
            'title' => $article->getTitle(),
            'updated' => $article->getUpdated(),
        ];
    }

    /**
     * @inheritDoc
     */
    public function save(CachedArticle $article, $relatedPost = null)
    {
        global $wpdb;
        $query = "SELECT `id` FROM `{$this->tableName}` WHERE `id` = %d LIMIT 1;";
        $query = $wpdb->prepare($query, $article->getId());
        $id = $wpdb->get_var($query);

        if ($id !== null) {
            $this->update($article);
        } else {
            $this->insert($article);
        }

        if (!$relatedPost) {
            return;
        }

        if (!$relatedPost->getId()) {
            throw new \DomainException('Save post first');
        }

        $syncTime = $relatedPost->getSyncTime() === null ? 0 : $relatedPost->getSyncTime();
        $query = "SELECT * FROM `{$this->junctionTableName}` WHERE `article_id` = %d AND `post_id` = %d LIMIT 1;";
        $query = $wpdb->prepare($query, $article->getId(), $relatedPost->getId());
        $row = $wpdb->get_row($query, ARRAY_A);

        if ($row !== null) {
            $wpdb->update(
                $this->junctionTableName,
                ['sync_time' => $syncTime],
                ['id' => $row['id']],
                ['%d'],
                ['%d']
            );
        } else {
            $wpdb->insert(
                $this->junctionTableName,
                [
                    'article_id' => $article->getId(),
                    'post_id' => $relatedPost->getId(),
                    'sync_time' => $syncTime,
                ],
                ['%d', '%d', '%d']
            );
        }
    }

    public function getById($id)
    {
        global $wpdb;
        $query = "SELECT * FROM `{$this->tableName}` WHERE `id` = %d LIMIT 1;";
        $query = $wpdb->prepare($query, $id);
        $row = $wpdb->get_row($query, ARRAY_A);

        if (null === $row) {
            throw new CachedArticleNotFoundException;
        }

        return $this->fromRow($row);
    }
}
