<?php

namespace EasyContent\WordPress\Implementation\Form;

use EasyContent\WordPress\Implementation\Form\Defaults\DefaultsProvider;
use EasyContent\WordPress\Implementation\Form\Exception\EmptyConfigException;
use EasyContent\WordPress\Implementation\Form\Exception\FormFieldMissingException;

abstract class BaseForm
{
    protected $fieldsConfig = [];

    private $fieldsValues = [];
    private $wasLoaded = false;

    final public function __construct(DefaultsProvider $defaults)
    {
        if (!$this->fieldsConfig) {
            throw new EmptyConfigException;
        }

        foreach ($this->fieldsConfig as $fieldName => $config) {
            $this->setFieldValue($fieldName, $defaults->get($fieldName));
        }
    }

    /**
     * @param string $fieldName
     * @return mixed
     * @throws FormFieldMissingException
     */
    final protected function getFieldValue($fieldName)
    {
        if (!array_key_exists($fieldName, $this->fieldsValues)) {
            throw new FormFieldMissingException;
        }

        return $this->fieldsValues[$fieldName];
    }

    /**
     * @param string $fieldName
     * @param mixed $value
     * @return void
     */
    final protected function setFieldValue($fieldName, $value)
    {
        $this->fieldsValues[$fieldName] = $value;
    }

    private function load(array $data)
    {
        if (!array_key_exists($this->getName(), $data)) {
            return;
        }

        $payload = $data[$this->getName()];

        foreach ($this->fieldsConfig as $fieldName => $config) {
            if (!array_key_exists($fieldName, $payload)) {
                continue;
            }

            $value = isset($config['filters'])
                ? $this->filter($config['filters'], $payload[$fieldName])
                : $payload[$fieldName];

            $this->setFieldValue($fieldName, $value);

            $this->wasLoaded = true;
        }
    }

    /**
     * @param callable|callable[] $filters
     * @param mixed $value
     * @return mixed
     */
    private function filter($filters, $value)
    {
        if (is_callable($filters)) {
            return call_user_func($filters, $value);
        }

        if (is_array($filters)) {
            foreach ($filters as $filter) {
                if (is_callable($filter)) {
                    $value = call_user_func($filter, $value);
                }
            }
        }

        return $value;
    }

    final public function loadFromPost()
    {
        $this->load($_POST);
    }

    final public function loadFromGet()
    {
        $this->load($_GET);
    }

    /**
     * @return bool
     */
    final public function wasLoaded()
    {
        return $this->wasLoaded;
    }

    /**
     * @return string
     */
    abstract public function getName();
}
