<?php

namespace EasyContent\WordPress;

use EasyContent\WordPress\Db\DbController;
use EasyContent\WordPress\Implementation\Container;
use EasyContent\WordPress\Metaboxes\MetaBoxesManager;
use EasyContent\WordPress\Notices\AdminNotice;
use EasyContent\WordPress\Notices\AdminNoticeInterface;
use EasyContent\WordPress\Notices\AdminNoticesService;
use EasyContent\WordPress\Pages\PagesManager;
use EasyContent\WordPress\Rest\Actions\GetArticleDetails;
use EasyContent\WordPress\Rest\Actions\GetCategories;
use EasyContent\WordPress\Rest\Actions\GetTags;
use EasyContent\WordPress\Rest\Actions\GetWorkflowStages;
use EasyContent\WordPress\Rest\Actions\PublishArticle;
use EasyContent\WordPress\Rest\Actions\PullArticle;
use EasyContent\WordPress\Rest\Actions\PushPost;
use EasyContent\WordPress\Rest\Actions\SetArticleWorkflowStage;
use EasyContent\WordPress\Rest\Actions\UnlinkPost;
use EasyContent\WordPress\Rest\RestManager;
use EasyContent\WordPress\Storages\ImageProcessingSettings;
use EasyContent\WordPress\Storages\PluginSettings;
use EasyContent\WordPress\Storages\PluginMisc;
use EasyContent\WordPress\Storages\StorageManager;

final class Plugin extends Singleton
{
    /** @var NoticesManager $noticesManager */
    public $noticesManager;

    /** @var DbController $dbController */
    private $dbController;

    /** @var bool $initialized */
    private $initialized = false;

    /** @var StorageManager $storageManager */
    private $storageManager;

    /** @var AdminNoticesService $adminNoticesService */
    private $adminNoticesService;

    /** @var Container $container */
    private $container;


    public function init()
    {
        if ( $this->initialized ) {
            return;
        }

        $this->adminNoticesService = new AdminNoticesService();
        $this->checkDatabaseIntegrity();

        $this->noticesManager = new NoticesManager();
        $this->storageManager = new StorageManager( [
            'settings' => new PluginSettings(),
            'misc' => new PluginMisc(),
            'image-processing' => new ImageProcessingSettings()
        ] );

        new PagesManager;

        new RestManager(RestManager::EASYCONTENT_V1_NAMESPACE, [
            '/categories'                        => GetCategories::class, // GET
            '/tags'                              => GetTags::class, // GET
            '/posts'                             => PublishArticle::class, // POST
            '/stages'                            => GetWorkflowStages::class, // GET
            '/posts/(?P<post_id>\d+)'            => SetArticleWorkflowStage::class, // PATCH
            '/posts/(?P<post_id>\d+)/push'       => PushPost::class, // POST
            '/posts/(?P<post_id>\d+)/unlink'     => UnlinkPost::class, // POST
            '/posts/(?P<post_id>\d+)/article'    => GetArticleDetails::class, // GET
            '/articles/(?P<article_id>\d+)/pull' => PullArticle::class // POST
        ], $this->getContainer());
        new ListTableManager();

        new MetaBoxesManager($this->getContainer());

        register_activation_hook( EASYCONTENT_ENTRY_FILE, [$this->getDatabaseController(), 'migrate'] );
        register_deactivation_hook( EASYCONTENT_ENTRY_FILE, [$this, 'onDeactivate'] );
        register_uninstall_hook( EASYCONTENT_ENTRY_FILE, [static::class, 'onUninstall'] );

        add_action('plugins_loaded', [$this->getDatabaseController(), 'migrate']);
        add_action( 'init', [$this, 'loadTextDomain'] );

        add_action( 'admin_enqueue_scripts', [$this, 'enqueueAdminAssets'] );

        add_action( 'delete_post', [Post::class, 'onDelete'], 10, 1 );
        add_action( 'save_post', [Post::class, 'onSave'], 10000, 3 );



        add_action( 'init', function() {
            $gutenbergCssSrc = EASYCONTENT_URL . 'assets/css/gutenberg' . (EASYCONTENT_DEBUG ? '' : '.min') . '.css';

            wp_register_style(
                'ec-gutenberg',
                $gutenbergCssSrc
            );

            wp_register_script(
                'ec-gutenberg',
                EASYCONTENT_URL . 'assets/js/gutenberg/build/index.js',
                ['wp-plugins', 'wp-edit-post', 'wp-element']
            );
        } );

        add_action( 'enqueue_block_editor_assets', function() {
            wp_enqueue_style( 'ec-gutenberg' );
            wp_enqueue_script( 'ec-gutenberg' );
        } );

        add_filter( 'is_protected_meta', function($protected, $metaKey, $metaType) {
            $allowed = ['_aioseop_title', '_aioseop_description', '_yoast_wpseo_title', '_yoast_wpseo_metadesc'];

            return in_array( $metaKey, $allowed ) ? false : $protected;
        }, 10000, 3 );

        // This filter allows us to insert post with empty content and insert modified data after that
        add_filter( 'wp_insert_post_empty_content', function($maybeEmpty, $postArray) {
            return false;
        }, 10, 2 );

        $this->initialized = true;
    }


    public function loadTextDomain()
    {
        $path = 'easycontent-wp/languages';
        load_plugin_textdomain( EASYCONTENT_TXTDOMAIN, false, $path );
    }


    public function enqueueAdminAssets($hookSuffix)
    {
        $pluginCssSrc = EASYCONTENT_URL . 'assets/css/plugin' . (EASYCONTENT_DEBUG ? '' : '.min') . '.css';

        wp_enqueue_style( 'ec-style', $pluginCssSrc, [], EASYCONTENT_PLUGIN_VERSION );

        if ( 'easycontent_page_easycontent-publishing-options' === $hookSuffix
            || 'post.php' === $hookSuffix
            || 'post-new.php' === $hookSuffix
        ) {
            $src = EASYCONTENT_URL . 'assets/js/admin' . (EASYCONTENT_DEBUG ? '' : '.min') . '.js';

            // jQuery UI is used to provide backward compatibility with classic editor
            wp_enqueue_style( 'ec-jquery-ui', EASYCONTENT_URL . 'assets/css/vendor/jquery-ui/jquery-ui.min.css', [], EASYCONTENT_PLUGIN_VERSION );
            wp_enqueue_script( 'ec-admin', $src, ['jquery', 'jquery-ui-dialog'], EASYCONTENT_PLUGIN_VERSION );
        }
    }

    /**
     * @return PluginSettings
     * @throws \Exception
     */
    public function getSettings()
    {
        return $this->storageManager->getStorage( 'settings' );
    }


    /**
     * @return PluginMisc
     * @throws \Exception
     */
    public function getMisc()
    {
        return $this->storageManager->getStorage( 'misc' );
    }


    /**
     * @return ImageProcessingSettings
     * @throws \Exception
     */
    public function getImageProcessingSettings()
    {
        return $this->storageManager->getStorage( 'image-processing' );
    }

    public function onDeactivate()
    {
        if ( ! EASYCONTENT_DEBUG ) {
            return;
        }

        $this->getDatabaseController()->clearDatabase();
    }


    public static function onUninstall()
    {
        Plugin::getInstance()->getDatabaseController()->clearDatabase();
    }


    /**
     * @return DbController
     */
    public function getDatabaseController()
    {
        if ( ! $this->dbController ) {
            $this->dbController = new DbController();
        }

        return $this->dbController;
    }

    public function getAdminNoticesService()
    {
        // Initialized in init() method
        return $this->adminNoticesService;
    }

    private function checkDatabaseIntegrity()
    {
        if ( $this->getDatabaseController()->checkIntegrity() ) {
            return;
        }

        add_action( 'init', function() {
            $easyContentAction = 'rerun-database-setup';
            $wpNonceAction = 'easycontent-rerun-database-setup';

            $action = isset( $_GET['easycontent-action'] )
                ? (string)$_GET['easycontent-action']
                : '';

            if ( $easyContentAction === $action && check_admin_referer( $wpNonceAction ) ) {
                $this->getDatabaseController()->clearDatabase();
                $this->getDatabaseController()->migrate();

                wp_redirect( admin_url() );
                exit;
            }



            $errorMessage = array_reduce( $this->getDatabaseController()->getIntegrityErrors(), function($carry, $item) {
                return $carry . ( $carry ? '<br>' : '' )
                    . '<strong>EasyContent:</strong> '
                    . esc_html( $item ) . '!';
            }, '' );

            $url = admin_url() . '?' . http_build_query( [
                '_wpnonce' => wp_create_nonce( $wpNonceAction ),
                'easycontent-action' => $easyContentAction
            ], '', '&', PHP_QUERY_RFC3986 );

            $errorMessage .= '<br>You can try to <a href="' . $url . '">rerun</a> database setup.';
            $errorMessage .= "<br><strong>WARNING!</strong> This action will erase all plugin's data from database!";

            $this->getAdminNoticesService()->addAdminNotice( new AdminNotice( $errorMessage, AdminNoticeInterface::TYPE_ERROR ) );
        } );
    }

    /**
     * @return Container
     */
    public function getContainer()
    {
        if (!$this->container) {
            $config = require_once(__DIR__ . DIRECTORY_SEPARATOR . 'dependencies-config.php');
            $this->container = new Container($config);
        }

        return $this->container;
    }
}
