<?php

namespace EasyContent\WordPress\Source\Api\Entity;

use EasyContent\WordPress\Source\Entity\CachedArticle;
use EasyContent\WordPress\Source\Entity\Post;

final class ImportedArticle extends BaseArticle
{
    /** @var string $excerpt */
    private $excerpt;
    /** @var string $customSlug */
    private $customSlug;
    /** @var string $content */
    private $content;
    /** @var null|FeaturedImage $featuredImage */
    private $featuredImage;

    /** @var null|Post $relatedPost Post entity. This data should be retrieved from WordPress */
    private $relatedPost;
    /** @var null|CachedArticle $cached */
    private $cached;

    public static function fromState(array $data)
    {
        return parent::fromState($data)
            ->setExcerpt($data)
            ->setCustomSlug($data)
            ->setContent($data)
            ->setFeaturedImage($data)
            ->setRelatedPostInternal($data)
            ->setCached($data);
    }

    /**
     * @param Post $post
     * @param int $statusId
     * @param array|int[] $categoriesIds
     * @return ImportedArticle
     */
    public static function fromPost(Post $post, $statusId, array $categoriesIds)
    {
        $article = new self;

        $article->id = $post->getRelatedArticleId();
        $article->currentWorkflowStatusId = $statusId;
        $article->topicName = $post->getTitle();
        $article->updated = time();
        $article->categoriesIds = $categoriesIds;
        $article->title = $post->getTitle();
        $article->metaTitle = $post->getMetaTitle();
        $article->metaDescription = $post->getMetaDescription();

        $article->excerpt = $post->getExcerpt();
        $article->customSlug = $post->getCustomSlug();
        $article->content = $post->getContent();
        $article->featuredImage = $post->getFeaturedImage();

        $article->relatedPost = $post;
        // TODO: Deal with $cached

        return $article;
    }

    /**
     * @param int $id
     * @return void
     */
    public function setId($id)
    {
        if ($this->id !== null) {
            throw new \DomainException('Article already has an id');
        }

        $this->id = $id;
    }

    /**
     * @param int $id
     * @return void
     */
    public function setCurrentWorkflowStatusId($id)
    {
        $this->currentWorkflowStatusId = $id;
    }

    /**
     * @return string
     */
    public function getExcerpt()
    {
        return $this->excerpt;
    }

    /**
     * @param array $data
     * @return ImportedArticle
     */
    private function setExcerpt(array $data)
    {
        $this->ensureArrayKeyExists('excerpt', $data);
        $this->ensureString($data['excerpt']);
        $this->excerpt = $data['excerpt'];
        return $this;
    }

    /**
     * @return string
     */
    public function getCustomSlug()
    {
        return $this->customSlug;
    }

    /**
     * @param array $data
     * @return ImportedArticle
     */
    private function setCustomSlug(array $data)
    {
        $this->ensureArrayKeyExists('custom_slug', $data);
        $this->ensureString($data['custom_slug']);
        $this->customSlug = $data['custom_slug'];
        return $this;
    }

    /**
     * @return string
     */
    public function getContent()
    {
        return $this->content;
    }

    /**
     * @param array $data
     * @return ImportedArticle
     */
    private function setContent(array $data)
    {
        $this->ensureArrayKeyExists('content', $data);
        $this->ensureString($data['content']);
        $this->content = $data['content'];
        return $this;
    }

    /**
     * @return FeaturedImage|null
     */
    public function getFeaturedImage()
    {
        return $this->featuredImage;
    }

    /**
     * @param array $data
     * @return ImportedArticle
     */
    private function setFeaturedImage(array $data)
    {
        $this->ensureArrayKeyExists('featured_image', $data);

        if ($data['featured_image']) {
            $this->featuredImage = FeaturedImage::fromState($data['featured_image']);
        }

        return $this;
    }

    /**
     * @param array $data
     * @return ImportedArticle
     */
    private function setRelatedPostInternal(array $data)
    {
        $this->ensureArrayKeyExists('related_post', $data);

        if (!$data['related_post']) {
            return $this;
        }

        $this->relatedPost = Post::fromState($data['related_post'], $this->getId());

        return $this;
    }

    /**
     * @param array $data
     * @return ImportedArticle
     */
    private function setCached(array $data)
    {
        $this->ensureArrayKeyExists('cached', $data);

        if (!$data['cached']) {
            return $this;
        }

        $this->cached = CachedArticle::fromState($data['cached']);

        return $this;
    }

    /**
     * @return Post|null
     */
    public function getRelatedPost()
    {
        return $this->relatedPost;
    }

    /**
     * @return int|null
     */
    public function getRelatedPostId()
    {
        return $this->relatedPost ? $this->relatedPost->getId() : null;
    }

    public function setRelatedPost(Post $relatedPost)
    {
        if ($this->relatedPost !== null) {
            throw new \DomainException('Article already has related post. Unable to reset.');
        }

        $this->relatedPost = $relatedPost;
    }

    /**
     * @return CachedArticle|null
     */
    public function getCached()
    {
        return $this->cached;
    }

    public function updateLocalCache()
    {
        if ($this->cached) {
            $this->cached->updateFromImported($this);
            return;
        }

        $this->cached = CachedArticle::fromImported($this);
    }
}
