<?php

namespace EasyContent\WordPress\Source\Entity;

use EasyContent\WordPress\Source\Api\Entity\Article;
use EasyContent\WordPress\Source\Api\Entity\Entity;
use EasyContent\WordPress\Source\Api\Entity\EntityInterface;
use EasyContent\WordPress\Source\Api\Entity\ImportedArticle;

final class CachedArticle extends Entity implements EntityInterface
{
    /** @var null|int $id */
    private $id;
    /** @var int $currentWorkflowStatusId */
    private $currentWorkflowStatusId;
    /** @var string $topicName */
    private $topicName;
    /** @var null|string $title */
    private $title;
    /** @var null|int $updated */
    private $updated;

    private function __construct() { }

    /**
     * @param ImportedArticle $article
     * @return self
     */
    public static function fromImported(ImportedArticle $article)
    {
        $cached = new self;

        $cached->id = $article->getId();
        $cached->currentWorkflowStatusId = $article->getCurrentWorkflowStatusId();
        $cached->topicName = $article->getTopicName();
        $cached->title = $article->getTitle();
        $cached->updated = $article->getUpdated();

        return $cached;
    }

    public static function fromSimple(Article $article)
    {
        $cached = new self;

        $cached->id = $article->getId();
        $cached->currentWorkflowStatusId = $article->getCurrentWorkflowStatusId();
        $cached->topicName = $article->getTopicName();
        $cached->title = $article->getTitle();
        $cached->updated = $article->getUpdated();

        return $cached;
    }

    /**
     * @param array $data
     * @return CachedArticle
     */
    public static function fromState(array $data)
    {
        return (new self)
            ->setId($data)
            ->setCurrentWorkflowStatusId($data)
            ->setTopicName($data)
            ->setTitle($data)
            ->setUpdated($data);
    }

    /**
     * @param array $data
     * @return CachedArticle
     */
    public function setId(array $data)
    {
        $this->ensureArrayKeyExists('id', $data);
        $this->ensureInteger($data['id']);

        $this->id = $data['id'];

        return $this;
    }

    /**
     * @param array $data
     * @return CachedArticle
     */
    private function setCurrentWorkflowStatusId(array $data)
    {
        $key = 'current_workflow_stage_id';
        $this->ensureArrayKeyExists($key, $data);
        $this->ensureInteger($data[$key]);

        $this->currentWorkflowStatusId = $data[$key];

        return $this;
    }

    /**
     * @param array $data
     * @return CachedArticle
     */
    private function setTopicName(array $data)
    {
        $this->ensureArrayKeyExists('topic_name', $data);
        $this->ensureString($data['topic_name']);

        $this->topicName = $data['topic_name'];

        return $this;
    }

    /**
     * @param array $data
     * @return CachedArticle
     */
    private function setTitle(array $data)
    {
        $this->ensureArrayKeyExists('title', $data);

        if ($data['title'] === null) {
            $this->title = null;
            return $this;
        }

        $this->ensureString($data['title']);

        $this->title = $data['title'];

        return $this;
    }

    /**
     * @param array $data
     * @return CachedArticle
     */
    private function setUpdated(array $data)
    {
        $this->ensureArrayKeyExists('updated', $data);

        if ($data['updated'] === null) {
            $this->updated = null;
            return $this;
        }

        $this->ensureInteger($data['updated']);

        $this->updated = $data['updated'];

        return $this;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getCurrentWorkflowStatusId()
    {
        return $this->currentWorkflowStatusId;
    }

    /**
     * @return string
     */
    public function getTopicName()
    {
        return $this->topicName;
    }

    /**
     * @return string|null
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @return int|null
     */
    public function getUpdated()
    {
        return $this->updated;
    }

    /**
     * @param ImportedArticle $article
     * @return void
     */
    public function updateFromImported(ImportedArticle $article)
    {
        $this->currentWorkflowStatusId = $article->getCurrentWorkflowStatusId();
        $this->topicName = $article->getTopicName();
        $this->title = $article->getTitle();
        $this->updated = $article->getUpdated();
    }

    /**
     * @param self $item
     * @return bool
     */
    public function fullyEquals($item)
    {
        return $this->id === $item->id
            && $this->currentWorkflowStatusId === $item->currentWorkflowStatusId
            && $this->topicName === $item->topicName
            && $this->title === $item->title
            && $this->updated === $item->updated;
    }
}
