<?php

namespace EasyContent\WordPress\Source\Entity;

use EasyContent\WordPress\Source\Api\Entity\Entity;
use EasyContent\WordPress\Source\Api\Entity\FeaturedImage;
use EasyContent\WordPress\Source\Api\Entity\ImportedArticle;
use EasyContent\WordPress\Source\Entity\Article\Extra;
use EasyContent\WordPress\Source\Entity\Post\PostType;
use EasyContent\WordPress\Source\Services\CategoriesMapper;

final class Post extends Entity
{
    /** @var null|int $id */
    private $id;
    /** @var int $authorId */
    private $authorId;
    /** @var string $status */
    private $status;
    /** @var string $type */
    private $type;
    /** @var string $commentStatus */
    private $commentStatus;
    /** @var string $title */
    private $title;
    /** @var string $excerpt */
    private $excerpt;
    /** @var string $customSlug post_name in WP */
    private $customSlug;
    /** @var string $content */
    private $content;
    /** @var array|int[] $categoriesIds */
    private $categoriesIds = [];
    /** @var array|int[] $tagsIds */
    private $tagsIds = [];
    /** @var null|FeaturedImage $featuredImage */
    private $featuredImage;
    /** @var string $metaTitle */
    private $metaTitle;
    /** @var string $metaDescription */
    private $metaDescription;
    /** @var null|string $date */
    private $date;

    /** @var null|int $relatedArticleId */
    private $relatedArticleId;
    /** @var null|int $syncTime */
    private $syncTime;

    private function __construct() { }

    /**
     * @param PluginSettings $settings
     * @param ImportedArticle $article
     * @param null|Extra $extra
     * @return Post
     */
    public static function create(PluginSettings $settings, ImportedArticle $article, $extra = null)
    {
        $post = new Post;
        $post->relatedArticleId = $article->getId();
        $post->setFields($article, $settings, $extra);

        return $post;
    }

    /**
     * @param array $data
     * @param null|int $relatedArticleId
     * @return self
     */
    public static function fromState(array $data, $relatedArticleId = null)
    {
        // TODO: Add $syncTime initialization
        $post = new Post;
        $post->relatedArticleId = $relatedArticleId;
        return $post
            ->setIdInternal($data)
            ->setAuthorId($data)
            ->setStatus($data)
            ->setType($data)
            ->setCommentStatus($data)
            ->setTitle($data)
            ->setExcerpt($data)
            ->setCustomSlug($data)
            ->setContent($data)
            ->setCategoriesIds($data)
            ->setFeaturedImage($data)
            ->setMetaTitle($data)
            ->setMetaDescription($data)
            ->setTagsIds($data)
            ->setDate($data);
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setIdInternal(array $data)
    {
        $this->ensureArrayKeyExists('ID', $data);
        $this->ensureInteger($data['ID']);

        $this->id = $data['ID'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setAuthorId(array $data)
    {
        $this->ensureArrayKeyExists('post_author', $data);
        $this->ensureInteger($data['post_author']);

        $this->authorId = $data['post_author'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setStatus(array $data)
    {
        $this->ensureArrayKeyExists('post_status', $data);
        $this->ensureString($data['post_status']);

        $this->status = $data['post_status'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setType(array $data)
    {
        $this->ensureArrayKeyExists('post_type', $data);
        $this->ensureString($data['post_type']);

        $this->type = $data['post_type'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setCommentStatus(array $data)
    {
        $this->ensureArrayKeyExists('comment_status', $data);
        $this->ensureString($data['comment_status']);

        $this->commentStatus = $data['comment_status'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setTitle(array $data)
    {
        $this->ensureArrayKeyExists('post_title', $data);
        $this->ensureString($data['post_title']);

        $this->title = $data['post_title'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setExcerpt(array $data)
    {
        $this->ensureArrayKeyExists('post_excerpt', $data);
        $this->ensureString($data['post_excerpt']);

        $this->excerpt = $data['post_excerpt'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setCustomSlug(array $data)
    {
        $this->ensureArrayKeyExists('post_name', $data);
        $this->ensureString($data['post_name']);

        $this->customSlug = $data['post_name'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setContent(array $data)
    {
        $this->ensureArrayKeyExists('post_content', $data);
        $this->ensureString($data['post_content']);

        $this->content = $data['post_content'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setCategoriesIds(array $data)
    {
        $this->ensureArrayKeyExists('categories_ids', $data);
        $this->ensureArray($data['categories_ids']);

        foreach ($data['categories_ids'] as $categoryId) {
            $this->ensureInteger($categoryId);
            $this->categoriesIds[] = $categoryId;
        }

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setFeaturedImage(array $data)
    {
        $this->ensureArrayKeyExists('featured_image', $data);

        if ($data['featured_image'] === null) {
            return $this;
        }

        $this->featuredImage = FeaturedImage::fromState($data['featured_image']);

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setMetaTitle(array $data)
    {
        $this->ensureArrayKeyExists('meta_title', $data);
        $this->ensureString($data['meta_title']);

        $this->metaTitle = $data['meta_title'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setMetaDescription(array $data)
    {
        $this->ensureArrayKeyExists('meta_description', $data);
        $this->ensureString($data['meta_description']);

        $this->metaDescription = $data['meta_description'];

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setTagsIds(array $data)
    {
        $this->ensureArrayKeyExists('tags', $data);
        $this->ensureArray($data['tags']);

        foreach ($data['tags'] as $tagId) {
            $this->ensureInteger($tagId);
            $this->tagsIds[] = $tagId;
        }

        return $this;
    }

    /**
     * @param array $data
     * @return Post
     */
    private function setDate(array $data)
    {
        $this->ensureArrayKeyExists('post_date', $data);
        $this->ensureString($data['post_date']);

        $this->date = $data['post_date'];

        return $this;
    }

    /**
     * @param ImportedArticle $article
     * @param PluginSettings $settings
     * @param null|Extra $extra
     * @return void
     */
    private function setFields(ImportedArticle $article, PluginSettings $settings, $extra = null)
    {
        $this->title           = $article->getTitle();
        $this->excerpt         = $article->getExcerpt();
        $this->customSlug      = $article->getCustomSlug();
        $this->content         = $article->getContent();
        $this->featuredImage   = $article->getFeaturedImage();
        $this->metaTitle       = $article->getMetaTitle();
        $this->metaDescription = $article->getMetaDescription();

        $this->authorId      = $settings->getPostAuthorId();
        $this->commentStatus = $settings->getCommentsStatus()->asString();

        $this->status = $extra
            ? $extra->getPostStatus()->asString()
            : $settings->getPostStatus()->asString();
        $this->type = $extra
            ? $extra->getPostType()->asString()
            : $settings->getPostType()->asString();

        $isPost = $extra ? $extra->getPostType()->isPost() : $settings->getPostType()->isPost();

        if ($isPost) {
            if ($extra) {
                if ($extra->getCategories()) {
                    $this->categoriesIds = $extra->getCategories();
                }

                if ($extra->getTags()) {
                    $this->tagsIds = $extra->getTags();
                }
            } elseif ($mappedCategories = CategoriesMapper::wpFromEcCategoriesIds($settings, $article->getCategoriesIds())) {
                $this->categoriesIds = $mappedCategories;
            }
        }

        if ($extra && $extra->getDate()) {
            $this->date = $extra->getDate()->format('Y-m-d H:i:s');
        }

        $this->syncTime = time();
    }

    /**
     * @param ImportedArticle $article
     * @param PluginSettings $settings
     * @param null|Extra $extra
     * @return void
     */
    public function update(ImportedArticle $article, PluginSettings $settings, $extra = null)
    {
        if (!$this->id) {
            throw new \DomainException('Post needs to be saved before update');
        }

        if (!$this->relatedArticleId) {
            throw new \DomainException("Post {$this->id} has no related article");
        }

        if ($this->relatedArticleId !== $article->getId() || $this->id !== $article->getRelatedPostId()) {
            $message = sprintf('Article "%s" and post "%s" are not related', $article->getId(), $this->id);
            throw new \DomainException($message);
        }

        $this->setFields($article, $settings, $extra);
    }

    public function attachArticle(ImportedArticle $article)
    {
        if ($this->relatedArticleId) {
            throw new \DomainException('Unable to attach article. This post already has related article.');
        }

        if (!$article->getId()) {
            throw new \DomainException('To attach article save it first');
        }

        $this->relatedArticleId = $article->getId();
        $this->syncTime = time();
    }

    public function detachArticle()
    {
        $this->relatedArticleId = null;
        $this->syncTime = null;
    }

    /**
     * @return int|null
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     * @return void
     * @throws \Exception
     */
    public function setId($id)
    {
        if ($this->id !== null) {
            throw new \Exception('This post already has an ID');
        }

        $this->id = $id;
    }

    /**
     * @return int
     */
    public function getAuthorId()
    {
        return $this->authorId;
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @return string
     */
    public function getCommentStatus()
    {
        return $this->commentStatus;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @return string
     */
    public function getExcerpt()
    {
        return $this->excerpt;
    }

    /**
     * @return string
     */
    public function getCustomSlug()
    {
        return $this->customSlug;
    }

    /**
     * @return string
     */
    public function getContent()
    {
        return $this->content;
    }

    /**
     * @return array|int[]
     */
    public function getCategoriesIds()
    {
        return $this->categoriesIds;
    }

    /**
     * @return FeaturedImage|null
     */
    public function getFeaturedImage()
    {
        return $this->featuredImage;
    }

    /**
     * @return string
     */
    public function getMetaTitle()
    {
        return $this->metaTitle;
    }

    /**
     * @return string
     */
    public function getMetaDescription()
    {
        return $this->metaDescription;
    }

    /**
     * Defines if WP post is 'post' and NOT 'page'
     * @return bool
     */
    public function isPost()
    {
        return $this->getType() === PostType::POST;
    }

    /**
     * @return array|int[]
     */
    public function getTagsIds()
    {
        return $this->tagsIds;
    }

    /**
     * @return string|null
     */
    public function getDate()
    {
        return $this->date;
    }

    /**
     * @return int|null
     */
    public function getRelatedArticleId()
    {
        return $this->relatedArticleId;
    }

    /**
     * @return int|null
     */
    public function getSyncTime()
    {
        return $this->syncTime;
    }
}
