<?php

namespace EasyContent\WordPress\Source\UseCase\Import;

use DomainException;
use EasyContent\WordPress\Source\Entity\CachedArticlesStorage;
use EasyContent\WordPress\Source\Entity\EasyContentArticlesStorage;
use EasyContent\WordPress\Source\Entity\PluginSettings;
use EasyContent\WordPress\Source\Entity\PluginStorage;
use EasyContent\WordPress\Source\Entity\Post;
use EasyContent\WordPress\Source\Entity\WorkflowStatusStorage;

final class Handler
{
    private $workflowStatuses;
    private $remoteArticles;
    private $cachedArticles;
    private $pluginSettings;
    private $pluginStorage;

    public function __construct(
        WorkflowStatusStorage $workflowStatuses,
        EasyContentArticlesStorage $remoteArticles,
        CachedArticlesStorage $cachedArticles,
        PluginSettings $pluginSettings,
        PluginStorage $pluginStorage
    ) {
        $this->workflowStatuses = $workflowStatuses;
        $this->remoteArticles = $remoteArticles;
        $this->cachedArticles = $cachedArticles;
        $this->pluginSettings = $pluginSettings;
        $this->pluginStorage = $pluginStorage;
    }

    public function handle(Command $command)
    {
        $newStatusId = $command->getNewWorkflowStatusId();

        if ($newStatusId !== null && !$this->workflowStatuses->existsById($newStatusId)) {
            $message = sprintf('Unable to find workflow stage with "%s" id.', $newStatusId);
            throw new DomainException($message);
        }

        $ids = $command->getArticlesIds(); // Used to track changes and to update cache in future
        $response = $this->remoteArticles->findByIds($ids);

        if (!$response) {
            $this->cachedArticles->removeByIds($ids);
            return;
        }

        foreach ($response->getArticles() as $article) {
            $updateRemoteStatus = false;

            if ($article->getRelatedPost()) {
                $article->getRelatedPost()->update($article, $this->pluginSettings);
            } else {
                $article->setRelatedPost(
                    Post::create($this->pluginSettings, $article)
                );
            }

            if ($newStatusId !== null && $newStatusId !== $article->getCurrentWorkflowStatusId()) {
                $article->setCurrentWorkflowStatusId($newStatusId);
                $updateRemoteStatus = true;
            }

            $article->updateLocalCache();

            $this->pluginStorage->save($article);

            if ($updateRemoteStatus) {
                $this->remoteArticles->updateStatus($article);
            }

            $key = array_search($article->getId(), $ids, true);
            unset($ids[$key]);
        }

        if ($ids) {
            $this->cachedArticles->removeByIds($ids);
        }
    }
}
