<?php

namespace EasyContent\WordPress\Db;

use EasyContent\WordPress\Db\Models\Article;
use EasyContent\WordPress\Db\Models\ArticlePost;
use EasyContent\WordPress\Db\Models\Category;
use EasyContent\WordPress\Db\Models\WorkflowStage;
use EasyContent\WordPress\Implementation\Db\Migrator;
use EasyContent\WordPress\Implementation\Entity\Db\Version;

final class DbController
{
    const DB_VERSION_OPTION_NAME = 'ec-current-db-version';

    private $integrityErrors = [];

    /** @var Version $currentDbVersion */
    private $currentDbVersion;
    /** @var Version $pluginDbVersion */
    private $pluginDbVersion;

    /** @var null|Migrator $migrator */
    private $migrator;

    public function __construct()
    {
        $result = get_option(self::DB_VERSION_OPTION_NAME);
        $optionMissingInDb = false === $result;

        $this->currentDbVersion = $optionMissingInDb
            ? Version::initial()
            : new Version($result); // Will throw an exception if value was corrupted

        if ($optionMissingInDb) {
            $this->saveVersion($this->currentDbVersion);
        }

        $this->pluginDbVersion = new Version(EASYCONTENT_DB_VERSION);
    }

    /**
     * @return Migrator
     */
    private function getMigrator()
    {
        if (!$this->migrator) {
            return $this->migrator = new Migrator($this->currentDbVersion, $this->pluginDbVersion);
        }

        return $this->migrator;
    }

    public function migrate()
    {
        $needMigration = $this->pluginDbVersion->greaterThan($this->currentDbVersion);

        if (!$needMigration) {
            return;
        }

        $this->getMigrator()->migrate();

        $this->currentDbVersion = $this->pluginDbVersion;
        $this->saveVersion($this->currentDbVersion);
    }

    /**
     * @param Version $version
     * @return void
     * @throws \Exception
     */
    private function saveVersion($version)
    {
        $saved = update_option(self::DB_VERSION_OPTION_NAME, $version->asString(), true);

        if (!$saved) {
            throw new \Exception(__( 'Unable to save DB_VERSION, please try again later', EASYCONTENT_TXTDOMAIN ));
        }
    }

    public function clearDatabase()
    {
        Article::drop();
        ArticlePost::drop();
        Category::drop();
        WorkflowStage::drop();

        delete_option( 'ec-current-db-version' );
        delete_option( 'ec-is-plugin-connected' );
        delete_option( 'ec-public-api-key' );
    }

    public function getIntegrityErrors()
    {
        return $this->integrityErrors;
    }

    public function checkIntegrity()
    {
        $this->integrityErrors = [];

        $tables = [
            Article::class,
            ArticlePost::class,
            Category::class,
            WorkflowStage::class
        ];

        global $wpdb;

        foreach ( $tables as $tableClass ) {
            if ( ! call_user_func( [$tableClass, 'tableExists'] ) ) {
                $tableName = call_user_func( [$tableClass, 'tableName'] );
                $format = __( '`%s`.`%s` table is missing', EASYCONTENT_TXTDOMAIN );
                $this->integrityErrors[] = sprintf( $format, $wpdb->dbname, $tableName );
            }
        }

        return ! $this->integrityErrors;
    }
}
