<?php

namespace EasyContent\WordPress\Helpers;

use EasyContent\WordPress\Attachment;
use EasyContent\WordPress\MediaList;
use PhpGangsters\EasyContent\HtmlHandler\DataTypes\Url;
use PhpGangsters\EasyContent\HtmlHandler\DOM\Document;
use PhpGangsters\EasyContent\HtmlHandler\DOM\NodeAttribute;
use PhpGangsters\EasyContent\HtmlHandler\FileList;

class PostContentProcessor
{
    /** @var string $content */
    protected $content;

    /** @var int $postId */
    protected $postId;

    /** @var Document $document */
    protected $document;

    /** @var bool $isDocumentPrepared */
    protected $isDocumentPrepared = false;

    /** @var MediaList $mediaList */
    protected $mediaList;

    /** @var array|FileList $parsedFileList */
    protected $parsedFileList;


    public function __construct($content, $postId)
    {
        $this->content = $content;
        $this->postId  = $postId;
    }


    protected function getDocument()
    {
        if ( ! $this->document ) {
            $urlBase = EASYCONTENT_ENDPOINT === 'https://easycontent.io'
                ? Url::LIVE_BASE
                : ( EASYCONTENT_ENDPOINT === 'https://cheesycontent.com'
                    ? Url::DEV_BASE
                    : Url::LOCAL_BASE
                );

            $this->document = new Document( '', '', $urlBase );
        }

        return $this->document;
    }


    protected function prepareDocument()
    {
        if ( $this->isDocumentPrepared ) {
            return;
        }

        $document = $this->getDocument();

        $document->loadHTML( $this->content );
        $document->removeAnnotations();
        $document->parse();

        $this->isDocumentPrepared = true;
    }


    protected function getMediaList()
    {
        if ( ! $this->mediaList ) {
            $this->mediaList = new MediaList();
        }

        return $this->mediaList;
    }


    protected function initParsedFileList()
    {
        if ( null === $this->parsedFileList ) {
            if ( ! $this->isDocumentPrepared ) {
                $this->prepareDocument();
            }

            $this->parsedFileList = $this->getDocument()->getFilesList();
        }
    }


    protected function getParsedFileList()
    {
        $this->initParsedFileList();

        return $this->parsedFileList;
    }


    protected function setParsedFileListItemWordPressUrl($key, $wpUrl)
    {
        $this->initParsedFileList();

        $this->parsedFileList[$key]->setWPUrl( $wpUrl );
    }


    protected function downloadMissingFiles()
    {
        $filesToDownload = array_diff_key(
            $this->getParsedFileList()->getFileNames(),
            $this->getMediaList()->getSourceUrls()
        );

        foreach ( $filesToDownload as $url => $fileName ) {
            $downloadedAttachment = Attachment::createFromUrl( $url, $this->postId );

            if ( $downloadedAttachment ) {
                $wpUrl = $downloadedAttachment->getPostObject()->guid;
                $this->setParsedFileListItemWordPressUrl( $url, $wpUrl );
            }
        }
    }


    protected function updateDocumentUrls()
    {
        $nodes = $this->getDocument()->getNodes();
        $parsedFileList = $this->getParsedFileList();
        $downloadedFiles = $this->getMediaList()->getSourceUrls();
        $mediaFiles = $this->getMediaList()->getMediaFiles();

        foreach ( $nodes as $node ) {
            /** @var NodeAttribute $attribute */
            foreach ( $node->getAttributeList() as $attribute ) {
                $attributeValue = $attribute->getValue();

                foreach ( $attribute->getFileList() as $url => $file ) {
                    $newUrl = null;

                    if ( isset( $downloadedFiles[$url] ) ) {
                        $mediaId = $downloadedFiles[$url];

                        $newUrl = ! isset( $mediaFiles[$mediaId] )
                            ? null
                            : $mediaFiles[$mediaId]->guid;
                    } else {
                        $newUrl = $parsedFileList[$url]->getWPUrl();
                    }

                    if ( $newUrl ) {
                        $attributeValue = str_replace( $url, $newUrl, $attributeValue );
                    }
                }

                $node->getSourceNode()->setAttribute( $attribute->getName(), $attributeValue );
            }
        }
    }


    protected function updateModifiedFiles()
    {
        $mediaFiles = $this->getMediaList()->getMediaFiles();

        /**
         * getFileNames() method returns an array that looks like this:
         *
         * [
         *     'http://s3.aws.com/piggy.png' => 'piggy.png',
         *     'http://s3.aws.com/cat.jpg'   => 'cat.jpg',
         *     'http://s3.aws.com/dog.gif'   => 'dog.gif'
         * ]
         *
         * This array_intersect_key() function gets files that were already uploaded
         */

        $filesToUpdate = array_intersect_key(
            $this->getMediaList()->getSourceUrls(),
            $this->getParsedFileList()->getFileNames()
        );

        foreach ( $filesToUpdate as $url => $attachmentId ) {
            if ( ! isset( $mediaFiles[$attachmentId] ) ) {
                continue;
            }

            try {
                $attachmentObj = new Attachment( $mediaFiles[$attachmentId] );
            } catch ( \Exception $e ) {
                continue;
            }

            $attachmentObj->updateFromUrl( $url );
        }
    }


    public function run()
    {
        if ( ! $this->isDocumentPrepared ) {
            $this->prepareDocument();
        }

        $this->downloadMissingFiles();
        $this->updateDocumentUrls();
        $this->updateModifiedFiles();

        return $this->getDocument()->saveHTML();
    }
}
